@@ -11,7 +11,7 @@ class DatasetCSVExport:
 
         filters['app_uuid'] = app.uuid
 
-        self.csv_dir = os.path.join(settings.MEDIA_ROOT, str(app.uuid), 'exports')
+        self.csv_dir = os.path.join(app.media_base_path, 'exports')
         self.filepath =  os.path.join(self.csv_dir, 'datasets.csv')
         
         self.filters = filters
@@ -29,16 +29,16 @@ class DatasetCSVExport:
         if os.path.isfile(self.filepath):
             os.remove(self.filepath)
 
-        columns = ['client_id', 'client_platform']
+        columns = ['client_id', 'platform']
 
         uuid_to_label = {
             'client_id' : 'client_id',
-            'client_platform' : 'client_platform',
+            'platform' : 'platform',
         }
 
         field_classes = {
             'client_id' : 'CharField',
-            'client_platform' : 'CharField',
+            'platform' : 'CharField',
         }
 
         for dataset in self.get_queryset():
