@@ -1,11 +1,13 @@
 from django.conf import settings
 from django.contrib.gis.db import models
 from django.utils.translation import gettext_lazy as _
-from django.contrib.postgres.fields import JSONField
+from django.utils import timezone
 from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.fields import GenericRelation
 from django.db import connection
 
+from django.contrib.gis.db.models.functions import Centroid
+
 User = get_user_model()
 
 from django.contrib.gis.geos import GEOSGeometry
@@ -16,12 +18,14 @@ from localcosmos_server.taxonomy.generic import ModelWithTaxon
 
 from localcosmos_server.utils import datetime_from_cron
 
-from datetime import datetime, timezone, timedelta
+from djangorestframework_camel_case.util import underscoreize
 
 from PIL import Image, ImageOps
 
 import uuid, json, os
 
+from .json_schemas import OBSERVATION_FORM_SCHEMA
+
 
 # a list of usable dataset validation classes
 DATASET_VALIDATION_CLASSPATHS = getattr(settings, 'DATASET_VALIDATION_CLASSES', ())
@@ -58,6 +62,34 @@ for classpath in DATASET_VALIDATION_CLASSPATHS:
 COMPLETED_VALIDATION_STEP = 'completed'
 
 
+'''
+    ObservationForm
+'''
+
+class ObservationForm(models.Model):
+
+    uuid = models.UUIDField(default=uuid.uuid4, editable=False)
+    version = models.IntegerField()
+    definition = models.JSONField()
+
+    @classmethod
+    def get_json_schema(cls):
+        return OBSERVATION_FORM_SCHEMA
+
+    class Meta:
+        unique_together=('uuid', 'version')
+
+
+'''
+    MetaData
+'''
+
+class MetaData(models.Model):
+
+    observation_form = models.ForeignKey(ObservationForm, on_delete=models.PROTECT)
+    data = models.JSONField()
+
+
 '''
     Dataset
     - datasets have to be validated AFTER being saved, which means going through the validation routine
@@ -68,10 +100,15 @@ class Dataset(ModelWithTaxon):
 
     uuid = models.UUIDField(default=uuid.uuid4, unique=True, editable=False)
 
+    observation_form = models.ForeignKey(ObservationForm, on_delete=models.PROTECT)
+
     ### data
     # all data except the taxononmic inherited from ModelWithTaxon is stored in the json data column without schema
     # for quicker queries, some fields have their own (redundant) db columns below
-    data = JSONField()
+    data = models.JSONField()
+    
+
+    meta_data = models.ForeignKey(MetaData, null=True, on_delete=models.PROTECT)
 
     ### redundant fields for quick DB queries
     # geographic reference, useful for anycluster and quick GIS queries
@@ -79,7 +116,6 @@ class Dataset(ModelWithTaxon):
     geographic_reference = models.GeometryField(srid=3857, null=True, blank=True) # for other geometries
 
     # app reference, for filtering datasets on maps, no FK to not lose data if the app is deleted
-    # app_uuid is transmitted by django_road, settings editable=False breaks django-rest-framework
     app_uuid = models.UUIDField()
 
     # temporal reference, if it is a timestamp
@@ -92,6 +128,7 @@ class Dataset(ModelWithTaxon):
     # client_id is redundant, also occurs in data
     # client_id can never be changed
     client_id = models.CharField(max_length=255, editable=False)
+    platform = models.CharField(max_length=255, editable=False)
 
 
     ### fields for validation
@@ -99,13 +136,17 @@ class Dataset(ModelWithTaxon):
     validation_step = models.CharField(max_length=255, null=True)
 
     # a list of errors
-    validation_errors = JSONField(null=True)
+    validation_errors = models.JSONField(null=True)
     is_valid = models.BooleanField(default=False)
+
+
+    # fields for publication
+    is_published = models.BooleanField(default=True)
     
 
     ### flags that should not reside inside the data json because they can differ between client and server
     # following timestamps can differ between server and offline device
-    # do not use auto_now_add or auto_now as these values are always set by django_road in the clients
+    # do not use auto_now_add or auto_now as these values are always set in the clients
     created_at = models.DateTimeField(editable=False) # timestamp when the dataset has been created on any of the clients
     last_modified = models.DateTimeField(null=True) # timestamp when the dataset has been alteres on any of the clients
 
@@ -115,6 +156,7 @@ class Dataset(ModelWithTaxon):
     # - iterate over all remaining steps as defined in DatasetValidationRoutine
     # - if no steps are defined, mark the dataset as valid, and set validation_step to 'completed'
     ###############################################################################################################
+
         
     @property
     def validation_routine(self):
@@ -152,6 +194,7 @@ class Dataset(ModelWithTaxon):
                 
             else:
                 self.is_valid = True
+                self.is_published = True
                 self.validation_step = COMPLETED_VALIDATION_STEP
 
                 self.save()
@@ -184,7 +227,7 @@ class Dataset(ModelWithTaxon):
     '''
     def update_redundant_columns(self):
 
-        reported_values = self.data['dataset']['reported_values']
+        reported_values = self.data
 
         # never alter the user that is assigned
         # in rare cases the following can happen:
@@ -192,80 +235,76 @@ class Dataset(ModelWithTaxon):
         # - fetching the browser device uid failed
         # -> an unassigned device_uuid is used, the logged in user is linked to the sighting
         # fix this problem and alter self.data['client_id'] to match the users browser client
-        if not self.pk:
 
-            # fill self.client_id
-            client_id = reported_values['client_id']
-            self.client_id = client_id
 
         # assign a user to the observation - even if it the dataset is updated
         if not self.user and self.client_id:
             # try find a user in usermobiles
-            client_id = reported_values['client_id']
-            self.client_id = client_id
-            client = UserClients.objects.filter(client_id=client_id).first()
+            client = UserClients.objects.filter(client_id=self.client_id).first()
             if client:
                 self.user = client.user
 
         # AFTER assigning the user, use the browser client_id if platform is browser
         if not self.pk:
 
-            platform = reported_values['client_platform']
-
-            # use browser client_id if browser and self.user
-            # this is only possible if the dataset has been transmitted using django_road, which assigned a user
-            if platform == 'browser' and self.user:
+            if self.platform == 'browser' and self.user:
                     
                 client = UserClients.objects.filter(user=self.user, platform='browser').order_by('pk').first()
 
                 if client:
-
                     user_browser_client_id = client.client_id
                 
-                    if user_browser_client_id != reported_values['client_id']:
-                        self.data['dataset']['reported_values']['client_id'] = user_browser_client_id
+                    if user_browser_client_id != self.client_id:
                         self.client_id = user_browser_client_id
 
         
         # update taxon
         # use the provided observation form json
-
-        observation_form = self.data['dataset']['observation_form']
-        
-        taxon_field_uuid = observation_form['taxonomic_reference']
+        taxon_field_uuid = self.observation_form.definition['taxonomicReference']
 
         if taxon_field_uuid in reported_values and type(reported_values[taxon_field_uuid]) == dict:
-            taxon_json = reported_values[taxon_field_uuid]
-                
+            taxon_json_camel = reported_values[taxon_field_uuid]
+            taxon_json = underscoreize(taxon_json_camel, no_underscore_before_number=True)
+
             lazy_taxon = self.LazyTaxonClass(**taxon_json)
-            self.set_taxon(lazy_taxon)
+            self.set_taxon(lazy_taxon) 
         
         # update coordinates or geographic_reference
         # {"type": "Feature", "geometry": {"crs": {"type": "name", "properties": {"name": "EPSG:4326"}},
         # "type": "Point", "coordinates": [8.703575134277346, 55.84336786584161]}, "properties": {"accuracy": 1}}
         # if it is a point, use coordinates. Otherwise use geographic_reference
-        geographic_reference_field_uuid = self.data['dataset']['observation_form']['geographic_reference']
-        if geographic_reference_field_uuid in self.data['dataset']['reported_values']:
+        geographic_reference_field_uuid = self.observation_form.definition['geographicReference']
+        if geographic_reference_field_uuid in self.data:
+
+            reported_value = self.data[geographic_reference_field_uuid]
+
+            srid_str = reported_value['geometry']['crs']['properties']['name']
+            srid = int(srid_str.split(':')[-1])
+
+            geojson = json.dumps(reported_value['geometry'])
+            geos_geometry = GEOSGeometry(geojson, srid=srid)
 
-            reported_value = self.data['dataset']['reported_values'][geographic_reference_field_uuid]
             if reported_value['geometry']['type'] == 'Point':
 
-                longitude = reported_value['geometry']['coordinates'][0]
-                latitude = reported_value['geometry']['coordinates'][1]
-                coords = GEOSGeometry('POINT({0} {1})'.format(longitude, latitude), srid=4326)
+                #longitude = reported_value['geometry']['coordinates'][0]
+                #latitude = reported_value['geometry']['coordinates'][1]
+                #coords = GEOSGeometry('POINT({0} {1})'.format(longitude, latitude), srid=srid)
                 
-                self.coordinates = coords
-                self.geographic_reference = coords
+                self.coordinates = geos_geometry
+                self.geographic_reference = geos_geometry
 
+            elif reported_value['geometry']['type'] == 'Polygon':
+                
+                self.geographic_reference = geos_geometry
+                self.coordinates = self.geographic_reference.centroid
 
         # update temporal reference
-        temporal_reference_field_uuid = self.data['dataset']['observation_form']['temporal_reference']
+        temporal_reference_field_uuid = self.observation_form.definition['temporalReference']
         
-        if temporal_reference_field_uuid in self.data['dataset']['reported_values']:
+        if temporal_reference_field_uuid in self.data:
 
-            reported_value = self.data['dataset']['reported_values'][temporal_reference_field_uuid]
+            reported_value = self.data[temporal_reference_field_uuid]
 
-            # {"cron": {"type": "timestamp", "format": "unixtime", "timestamp": 1564566855177}, "type": "Temporal"}}
             if reported_value['cron']['type'] == 'timestamp' and reported_value['cron']['format'] == 'unixtime':
                 self.timestamp = datetime_from_cron(reported_value)
 
@@ -307,25 +346,22 @@ class Dataset(ModelWithTaxon):
         image = DatasetImages.objects.filter(dataset=self).first()
 
         if image:
-            return image.thumbnail()
+            url = image.get_image_url(250, square=True)
+            return url
         
         return None
 
     # this is not the validation routine, but the check for general localcosmos requirements
     def validate_requirements(self):
-        if self.data is None:
+        if not self.data:
             raise ValueError('Dataset needs at least some data')
 
-        reported_values = self.data['dataset']['reported_values']
-
-        if 'client_id' not in reported_values or reported_values['client_id'] == None or len(reported_values['client_id']) == 0:
-            raise ValueError('no client_id found in dataset.reported_values')
-
-
     def save(self, *args, **kwargs):
 
         created = False
         if not self.pk:
+            if not self.created_at:
+                self.created_at = timezone.now()
             created = True
 
         # validate the JSON
@@ -334,6 +370,9 @@ class Dataset(ModelWithTaxon):
         # update columns
         self.update_redundant_columns()
 
+        if settings.LOCALCOSMOS_SERVER_PUBLISH_INVALID_DATA == False:
+            self.is_published = False
+
         # this will run the validator
         super().save(*args, **kwargs)
 
@@ -343,7 +382,7 @@ class Dataset(ModelWithTaxon):
 
     def __str__(self):
         if self.taxon_latname:
-            return '%s' % (self.taxon_latname)
+            return '{}'.format(self.taxon_latname)
         return str(_('Unidentified'))
     
 
@@ -386,119 +425,114 @@ class DatasetValidationRoutine(ModelWithTaxon):
 
 
 
-def dataset_image_path(instance, filename):
-    return 'datasets/{0}/images/{1}/{2}'.format(str(instance.dataset.uuid), instance.field_uuid, filename)
 
 # Dataset Images have to be compatible with GenericForms
 # - reference the field uuid
-# - supply a thumbnail
+# - supply 1x 2x 4x image sizes
+# - the numbers represent the width of the large side
+IMAGE_SIZES = {
+    'regular' : {
+        '1x' : 250,
+        '2x' : 500,
+        '4x' : 1000,
+    },
+    'large' : {
+        '4x' : 1000,
+        '8x' : 2000,
+    },
+    'all' : {
+        '1x' : 250,
+        '2x' : 500,
+        '4x' : 1000,
+        '8x' : 2000,
+    }
+}
+
+def dataset_image_path(instance, filename):
+    return 'datasets/{0}/images/{1}/{2}'.format(str(instance.dataset.uuid), instance.field_uuid, filename)
+
+
 class DatasetImages(models.Model):
 
+    resized_folder_name = 'resized'
+
     dataset = models.ForeignKey(Dataset, on_delete=models.CASCADE)
     field_uuid = models.UUIDField()
     image = models.ImageField(max_length=255, upload_to=dataset_image_path)
 
+    created_at = models.DateTimeField(auto_now_add=True)
+
     @property
     def user(self):
         return self.dataset.user
 
-    def get_thumb_filename(self, size=100):
+    @property
+    def client_id(self):
+        return self.dataset.client_id
 
-        filename = os.path.basename(self.image.path)
-        blankname, ext = os.path.splitext(filename)
+    @property
+    def app_uuid(self):
+        return self.dataset.app_uuid
 
-        thumbname = '{0}-{1}{2}'.format(blankname, size, ext)
-        return thumbname
-
-
-    def get_thumbfolder(self):
+    @property
+    def resized_folder(self):
 
         folder_path = os.path.dirname(self.image.path)
         
-        thumbfolder = os.path.join(folder_path, 'thumbnails')
-        if not os.path.isdir(thumbfolder):
-            os.makedirs(thumbfolder)
+        resized_folder = os.path.join(folder_path, self.resized_folder_name)
+        if not os.path.isdir(resized_folder):
+            os.makedirs(resized_folder)
 
-        return thumbfolder
-    
-
-    def get_image_format(self, image):
-
-        if image.format:
-            image_format = image.format
-        else:
-            image_format = 'JPEG'
-
-        return image_format
-    
-
-    # thumbnails are always square
-    def thumbnail(self, size=100):
-
-        thumb_size = (size, size)
-
-        image_path = self.image.path
-        
-        thumbfolder = self.get_thumbfolder()
-
-        thumbname = self.get_thumb_filename(size)
-        thumbpath = os.path.join(thumbfolder, thumbname)
-
-        if not os.path.isfile(thumbpath):
-
-            imageFile = Image.open(image_path)
-
-            thumb_format = self.get_image_format(imageFile)
-
-            thumb = ImageOps.fit(imageFile, thumb_size, Image.BICUBIC)
-
-            thumb.save(thumbpath, thumb_format)
-        
-        thumburl = os.path.join(os.path.dirname(self.image.url), 'thumbnails', thumbname)
-        return thumburl
+        return resized_folder
 
 
-    def resized(self, name, max_size=[1920, 1080]):
-
-        thumbfolder = self.get_thumbfolder()
+    def get_resized_filename(self, size, square=False):
 
         filename = os.path.basename(self.image.path)
         blankname, ext = os.path.splitext(filename)
-        thumbname = '{0}-{1}{2}'.format(filename, name, ext)
 
-        thumbpath = os.path.join(thumbfolder, thumbname)
-
-        if max_size[0] <= max_size[1]:
-            short_edge = max_size[0]
-            long_edge = max_size[1]
+        if square:
+            filename = '{0}-{1}-square{2}'.format(blankname, size, ext)
         else:
-            short_edge = max_size[1]
-            long_edge = max_size[0]
+            filename = '{0}-{1}{2}'.format(blankname, size, ext)
+        return filename
 
-        if not os.path.isfile(thumbpath):
 
-            image_path = self.image.path
-            imageFile = Image.open(image_path)
+    def get_image_url(self, size, square=False):
 
-            if imageFile.width >= imageFile.height:
-                size = (long_edge, short_edge)
+        filename = self.get_resized_filename(size, square=square)
+
+        resized_path = os.path.join(self.resized_folder, filename)
+
+        if not os.path.isfile(resized_path):
+
+            max_size = (size, size)
+
+            image = Image.open(self.image.path)
+
+            if square:
+                image = ImageOps.fit(image, max_size, Image.BICUBIC)
             else:
-                size = (short_edge, long_edge)
+                image.thumbnail(max_size)
 
-            if imageFile.width > size[0] or imageFile.height > size[1]:
-                imageFile.thumbnail(size, Image.BICUBIC)
+            image.save(resized_path, image.format)
 
-            image_format = self.get_image_format(imageFile)
-                
-            imageFile.save(thumbpath, image_format)
+        image_url = os.path.join(os.path.dirname(self.image.url), self.resized_folder_name, filename)
 
-        thumburl = os.path.join(os.path.dirname(self.image.url), 'thumbnails', thumbname)
-        return thumburl
+        return image_url
 
+
+    @property
+    def image_urls(self):
+
+        image_urls = {}
         
-    # full hd image, 1920x1080 or 1080x1920
-    def full_hd(self):
-        return self.resized('fullhd', max_size=[1920,1080])
+        for size_name, image_size in IMAGE_SIZES['all'].items():
+            # create the resized image, respecting which side is the longer one
+            image_url = self.get_image_url(image_size)
+            image_urls[size_name] = image_url
+
+        return image_urls
 
 
     def __str__(self):
@@ -508,3 +542,12 @@ class DatasetImages(models.Model):
         return 'Dataset Image #{0}'.format(self.id)
     
         
+'''
+    USER Geometry
+    max 3 per user
+'''
+class UserGeometry(models.Model):
+
+    user = models.ForeignKey(User, on_delete=models.CASCADE)
+    geometry = models.GeometryField(srid=3857, null=True, blank=True)
+    name = models.CharField(max_length=355)
