@@ -24,7 +24,9 @@ class ServerControlPanelMiddleware:
 
             else:
 
-                if 'log-in' not in request.path:                
+                login_path = reverse('log_in')
+
+                if login_path not in request.path:                
                     url = '{0}?next={1}'.format(reverse('log_in'), request.path)
                     return redirect(url)
         
