@@ -6,13 +6,16 @@ from django.contrib.contenttypes.models import ContentType
 from django.utils.translation import gettext as _
 from django.urls import reverse
 
+from localcosmos_server.utils import get_taxon_search_url
 from localcosmos_server.taxonomy.forms import AddSingleTaxonForm, TypedTaxonomicRestrictionForm
 from localcosmos_server.models import TaxonomicRestriction
+from localcosmos_server.template_content.models import TemplateContent
+
 
 '''
     render taxonomic restrictions from app taxa
 '''
-def get_taxonomic_restriction_context(content, content_type, restrictions, taxon_search_url, typed):
+def get_taxonomic_restriction_context(app, content, content_type, restrictions, taxon_search_url, typed, action_url):
     
     form_class = AddSingleTaxonForm
     if typed == 'typed':
@@ -27,27 +30,48 @@ def get_taxonomic_restriction_context(content, content_type, restrictions, taxon
     }
 
     context = {
+        'app' : app,
         'form': form_class(**form_kwargs),
         'typed': typed,
         'restrictions': restrictions,
         'content' : content,
         'content_type' : content_type,
+        'action_url' : action_url,
     }
     return context
 
-    
-@register.inclusion_tag('localcosmos_server/taxonomic_restrictions_form.html')
-def render_app_taxonomic_restriction(app, content, typed=None):
-
-    taxon_search_url = reverse('search_app_taxon', kwargs={'app_uid':app.uid})
+# combined restrictions for app_kit and localcosmos_server
+@register.inclusion_tag('localcosmos_server/taxonomy/taxonomic_restrictions.html')
+def render_taxonomic_restriction(app, content, typed=None):
 
     content_type = ContentType.objects.get_for_model(content)
-    restrictions = TaxonomicRestriction.objects.filter(
+
+    url_kwargs = {
+        'content_type_id' : content_type.id,
+        'object_id' : content.id,
+    }
+
+    if typed == 'typed':
+        url_kwargs['typed'] = 'typed'
+
+
+    if settings.LOCALCOSMOS_PRIVATE == True or content.__class__ == TemplateContent:
+        RestrictionModel = TaxonomicRestriction
+        url_kwargs['app_uid'] = app.uid
+        action_url = reverse('manage_app_taxonomic_restrictions', kwargs=url_kwargs)
+    else:
+        from app_kit.generic import AppContentTaxonomicRestriction
+        RestrictionModel = AppContentTaxonomicRestriction
+        action_url = reverse('add_taxonomic_restriction',  kwargs=url_kwargs)
+
+    taxon_search_url = get_taxon_search_url(app, content)
+    
+    restrictions = RestrictionModel.objects.filter(
         content_type=content_type,
         object_id=content.id,
     )
 
-    context = get_taxonomic_restriction_context(content, content_type, restrictions, taxon_search_url, typed)
+    context = get_taxonomic_restriction_context(app, content, content_type, restrictions, taxon_search_url, typed, action_url)
     return context
 
 
@@ -128,3 +152,14 @@ def render_serializer_field(field, style):
 def get_app_locale(context, key):
     language = context['request'].LANGUAGE_CODE
     return context['request'].app.get_locale(key, language)
+
+
+@register.simple_tag
+def content_image_url(instance, image_type):
+    content_image = instance.image(image_type)
+    return content_image.image_url()
+
+
+@register.filter
+def is_ajax(request):
+    return request.headers.get('x-requested-with') == 'XMLHttpRequest'
\ No newline at end of file
